-- This file is  free  software, which  comes  along  with  SmartEiffel. This
-- software  is  distributed  in the hope that it will be useful, but WITHOUT
-- ANY  WARRANTY;  without  even  the  implied warranty of MERCHANTABILITY or
-- FITNESS  FOR A PARTICULAR PURPOSE. You can modify it as you want, provided
-- this header is kept unaltered, and a notification of the changes is added.
-- You  are  allowed  to  redistribute  it and sell it, alone or as a part of
-- another product.
--       Copyright (C) 1994-2002 LORIA - INRIA - U.H.P. Nancy 1 - FRANCE
--          Philippe RIBET and Dominique COLNET - SmartEiffel@loria.fr
--                       http://SmartEiffel.loria.fr
--
class DRAW_STYLE
   -- DRAW_STYLE is low-level service who describe drawing attributes
   -- (color, line width, font...)


inherit
   GRAPHIC --TODO: remove? (needed for default_font)
   UNICODE_STRING_HANDLER
   DRAWABLE_HANDLER
   MEMORY redefine dispose end

create {ANY}
   default_create

feature {}
   default_create is
      do
	 style := basic_style_new
	 font := default_font
	 basic_style_set_font(style, font.font)
      end

feature {ANY}
   color: COLOR is
      do
	 --TODO: store on eiffel side or get values from C side ?
	 not_yet_implemented
      end

   set_color(c: COLOR) is
      require
	 c /= Void
      do
	 basic_style_set_color(style, c.storage)
      end

   line_width: INTEGER is
      do
	 --TODO: store on eiffel side or get values from C side ?
	 not_yet_implemented
      end

   set_line_width(w: INTEGER) is
      require
	 w > 0
      do
	 basic_style_set_line_width(style, w)
      end

feature {RECTANGLE}
   draw_rectangle(d: POINTER; x1, y1, w, h: INTEGER) is
      require
	 w > 0
	 h > 0
 	 d /= default_pointer
      do
	 basic_draw_rectangle(d, style, x1, y1, w, h)
      end

feature {FILL_RECTANGLE}
   draw_fill_rectangle(d: POINTER; x1, y1, w, h: INTEGER) is
      require
	 w > 0
	 h > 0
	 d /= default_pointer
      do
	 basic_draw_fill_rectangle(d, style, x1, y1, w, h)
      end

feature {LINE}
   draw_line(d: POINTER; x1, y1, x2, y2: INTEGER) is
      require
	 d /= default_pointer
      do
	 basic_draw_line(d, style, x1, y1, x2, y2)
      end

feature {LABEL}
   draw_string(d: POINTER; s: UNICODE_STRING; x, y: INTEGER) is
      require
	 s /= Void
	 d /= default_pointer
      do
	 --TODO: surrogate characters !
	 --TODO: combining characters !
	 basic_draw_text(d, style, x, y, font.base_line, s.storage.to_external, s.count)
      end

feature {}
   draw_point(d: POINTER; x1, y1: INTEGER) is
      require
	 d /= default_pointer
      do
	 basic_draw_point(d, style, x1, y1)
      end

   draw_arc(d: POINTER; x1, y1, w, h: INTEGER; angle1, angle2: DOUBLE) is
	 -- arc will be drawn inside the rectangle defined with x1, y1, w, h
      require
	 w > 0
	 h > 0
	 d /= default_pointer
      do
	 basic_draw_arc(d, style, x1, y1, w, h, angle1, angle2)
      end

   draw_arc_radius(d: POINTER; x, y, r1, r2: INTEGER; angle1,angle2: DOUBLE) is
	 -- arc will be drawn using (x, y) as center and r1/r2 as
	 -- horizontal/vertical radius
      require
	 r1 > 0
	 r2 > 0
	 d /= default_pointer
      do
	 basic_draw_arc(d, style, x - r1, y - r2, 2*r1, 2*r2, angle1, angle2)
      end

   draw_fill_arc(d: POINTER; x1, y1, w, h: INTEGER; angle1, angle2: DOUBLE) is
	 -- arc will be drawn inside the rectangle defined with x1, y1, w, h
      require
	 w > 0
	 h > 0
	 d /= default_pointer
      do
	 basic_draw_fill_arc(d, style, x1, y1, w, h, angle1, angle2)
      end

   draw_fill_arc_radius(d: POINTER; x, y, r1, r2: INTEGER; angle1, angle2: DOUBLE) is
	 -- arc will be drawn using (x, y) as center and r1/r2 as
	 -- horizontal/vertical radius
      require
	 r1 > 0
	 r2 > 0
	 d /= default_pointer
      do
	 basic_draw_arc(d, style, x - r1, y - r2, 2*r1, 2*r2, angle1, angle2)
      end

feature {}
   style: POINTER
   font: BASIC_FONT

   dispose is
      do
	 basic_style_free(style)
      end

   basic_style_new: POINTER is
      external "SmartEiffel"
      end

   basic_style_set_color(sty: POINTER; color_: POINTER) is
      external "SmartEiffel"
      end

   basic_style_set_line_width(sty: POINTER; w: INTEGER) is
      external "SmartEiffel"
      end

   basic_style_set_font(sty, f: POINTER) is
      external "SmartEiffel"
      end

   basic_style_free(sty: POINTER) is
      external "SmartEiffel"
      end

   basic_draw_point(draw, sty:POINTER; x1, y1: INTEGER) is
      external "SmartEiffel"
      end

   basic_draw_line(draw, sty:POINTER; x1, y1, x2, y2: INTEGER) is
      external "SmartEiffel"
      end

   basic_draw_rectangle(draw, sty:POINTER; x1, y1, w, h: INTEGER) is
      external "SmartEiffel"
      end

   basic_draw_arc(draw, sty:POINTER; x1, y1, w, h: INTEGER; a1, a2: DOUBLE) is
      external "SmartEiffel"
      end

   basic_draw_fill_rectangle(draw, sty:POINTER; x1, y1, w, h: INTEGER) is
      external "SmartEiffel"
      end

   basic_draw_fill_arc(draw, sty:POINTER; x1, y1, w, h: INTEGER; a1, a2: DOUBLE) is
      external "SmartEiffel"
      end

   basic_draw_text(draw, sty: POINTER; x, y, bl: INTEGER; text: POINTER; length: INTEGER) is
      external "SmartEiffel"
      end
end
