-- This file is  free  software, which  comes  along  with  SmartEiffel. This
-- software  is  distributed  in the hope that it will be useful, but WITHOUT
-- ANY  WARRANTY;  without  even  the  implied warranty of MERCHANTABILITY or
-- FITNESS  FOR A PARTICULAR PURPOSE. You can modify it as you want, provided
-- this header is kept unaltered, and a notification of the changes is added.
-- You  are  allowed  to  redistribute  it and sell it, alone or as a part of
-- another product.
--       Copyright (C) 1994-2002 LORIA - INRIA - U.H.P. Nancy 1 - FRANCE
--          Philippe RIBET and Dominique COLNET - SmartEiffel@loria.fr
--                       http://SmartEiffel.loria.fr
--
deferred class IMAGE
   -- This class allow to draw an image in memory (process memory)
   -- and send it to the display. Each draw require lots of network
   -- if the display is remote. A draw require hard work to convert 
   -- the image into display format (be the display local or remote).
   -- See `PIXMAP' for image stored in the "display" memory.

inherit
   WIDGET
   DRAWABLE

feature {}
   widget_size_init(width_, height_: INTEGER) is
      require
	 width_ > 0
	 height_ > 0
      do
	 width := width_
	 height := height_
	 min_width := width
	 min_height := height
	 std_width := width
	 std_height := height


	 --create storage.make_filled('%/000/',width*height*4) --(depth//8))
	 count := width*height*4
	 storage := storage.calloc(count)
	 widget := basic_image_create(width_, height_, depth_, storage.to_external, $drawing_widget)
      end

feature {ANY}
   width: INTEGER
   height: INTEGER
   depth: INTEGER
   min_width: INTEGER
   min_height: INTEGER
   std_width: INTEGER
   std_height: INTEGER

   widget: POINTER

   storage: NATIVE_ARRAY[CHARACTER]
   count: INTEGER

   put(c: CHARACTER; i: INTEGER) is
	 -- Put `c' at position `index'.
      require
         valid_index: i.in_range(0,count-1)
      do
	 storage.put(c,i)
      end

feature {LAYOUT}
   expose_paint is
      do
	 basic_image_draw(parent.drawing_widget, drawing_widget, pos_x, pos_y, width, height);
      end

   set_geometry(x, y, w, h: INTEGER) is
      do
	 pos_x := x
	 pos_y := y
	 width := w
	 height := h
      end

feature {}
   basic_image_create(w,h,d: INTEGER; data: POINTER; drawable: POINTER): POINTER is
      external "SmartEiffel"
      end

   basic_image_draw(win: POINTER; img: POINTER; x, y, w, h: INTEGER) is
      external "SmartEiffel"
      end

end

