-- This file is  free  software, which  comes  along  with  SmartEiffel. This
-- software  is  distributed  in the hope that it will be useful, but WITHOUT
-- ANY  WARRANTY;  without  even  the  implied warranty of MERCHANTABILITY or
-- FITNESS  FOR A PARTICULAR PURPOSE. You can modify it as you want, provided
-- this header is kept unaltered, and a notification of the changes is added.
-- You  are  allowed  to  redistribute  it and sell it, alone or as a part of
-- another product.
--       Copyright (C) 1994-2002 LORIA - INRIA - U.H.P. Nancy 1 - FRANCE
--          Philippe RIBET and Dominique COLNET - SmartEiffel@loria.fr
--                       http://SmartEiffel.loria.fr
--
class TEXT_CURSOR_JOB
-- Draw blinking cursor in text area.

inherit
   PERIODIC_JOB
   GRAPHIC

creation {ANY}
   default_create

feature {}
   default_create is
      do
	 create draw_kit
	 priority := 100
	 period := 0.5
	 draw_cursor_proc := agent draw_cursor
      end


feature {ANY}
   start_in(text_field: TEXT_FIELD) is
      do
	 check window = Void end
	 window := text_field
	 window.when_expose(draw_cursor_proc)
	 vision.loop_stack.add_job(Current)
	 next_time.update
	 next_time := next_time + period
	 done := False
	 height := text_field.cursor_height
	 move
      end

   stop is
      do
	 if visible then
	    visible := False
	    update_cursor
	 end
	 if window /= Void then -- Double stop may occur when if button is down
	    window.when_expose_signal.disconnect(draw_cursor_proc)
	    window := Void
	 end
	 done := True
      end

   move is
      do
	 if pos_x < window.width and then pos_y + height <= window.height then
	    update_cursor
	 end
	 visible := True
	 next_time.update
	 next_time := next_time + period
	 pos_x := window.cursor_pos_x
	 pos_y := window.cursor_pos_y
      end

feature {LOOP_ITEM}
   done: BOOLEAN

   continue is
      do
	 if window /= Void and then not done then
	    visible := not visible
	    update_cursor
	 end
      end

   restart is
      do
	 done := window /= Void
      end

feature {}
   draw_kit: DRAW_KIT
   window: TEXT_FIELD
   visible: BOOLEAN
   pos_x, pos_y, height: INTEGER
   draw_cursor_proc: PROCEDURE[ANY, TUPLE]

   update_cursor is
      do
	 window.clear_area(pos_x, pos_y, 1, height)
      end

   draw_cursor is
      do
	 check window /= Void end
	 if visible then
	    draw_kit.set_drawable(window)
	    draw_kit.set_color(black_color)
	    draw_kit.line(pos_x, pos_y, pos_x, pos_y + height - 1)
	 end
      end

end
